// $Source$
// $Revision: 2187 $
// $Date: 2006-12-15 03:17:56 +0300 (Fri, 15 Dec 2006) $
// $Author: orlov $

#ifndef __rf_char_hh__
#define __rf_char_hh__

#include "rf_types.hh"
#include "rf_term.hh"
#include "rf_expr.hh"

namespace rftype
{

using namespace rfrt ;

class Char :
  public Term
{

public:
  ///
  /// Constructor from a wide character
  inline Char (wchar_t _wch) ;

#if 0
  inline Char(const Char& _ch) ;
#endif

  static Expr create_expr (wchar_t const* _wstr, size_t _len = (size_t)-1) ;

  static Expr create_expr (const WString& _wstr) ;
  static Expr create_expr (const wchar_t _ch) ;

  static Expr create_expr (char const* _str, char const* _locale = null) ;

  inline wchar_t to_wchar_t() const;

  ///
  /// Create new expression -- character sequence representing specified
  /// argument.
  static Expr to_chars (const Expr& _expr) ;

  ///
  /// Convert character sequence to lower case.  Note that argument must be a
  /// character sequence and the method doesn't check this.
  static Expr to_lower (const Expr& _expr) ;

  ///
  /// Convert character sequence to upper case.  Note that argument must be a
  /// character sequence and the method doesn't check this.
  static Expr to_upper (const Expr& _expr) ;

  static Expr bytes_to_chars (const Expr& _expr) ;
  static Expr chars_to_bytes (const Expr& _expr) ;
  //  static Expr to_int (const Expr& _expr) ;

  inline bool is_digit() const;

  inline bool is_letter() const;

  inline bool is_space() const;
  
#if 0
  inline Int to_byte() const;

  inline bool operator == (Char const&) const;


  inline wchar_t to_wchar_t() const;
#endif

  static inline bool is_flat () ;

  static pxx::WString to_string (Char const* _char) ;
  static short_type_funcs_t funcs ;
  static TypeRegister reg ;

  ///
  /// Term class for template instantiations
  static const term_class_t term_class = flat_term;
};

}

#endif // __rf_char_hh__
