// $Source$
// $Revision: 2270 $
// $Date: 2007-01-22 17:20:03 +0300 (Mon, 22 Jan 2007) $

#ifndef __rf_memory_chunk_hh__
#define __rf_memory_chunk_hh__

#include "pxx_heap_allocator.hh"
#include "pxx_heap_allocator.ih"
#include "rf_common.hh"

namespace rfrt {

using namespace pxx;

class Term;

class MemoryChunk
{

private:
  uintptr_t ref_count;

  NO_COPY_CTOR(MemoryChunk)
  NO_ASSIGN(MemoryChunk)

public:

  static MemoryChunk* create_instance (size_t _size)
  {
    MemoryChunk* p = static_cast<MemoryChunk*>(allocator->allocate(_size));
    if (p == null) FATAL("Unable to allocate memory");
    p->ref_count = 1;
    return p;
  }

  static void destroy_instance (void* _ptr)
  {
    allocator->deallocate(_ptr);
  }

  //
  // return size of usable memory
  inline uintptr_t get_size() const;
  //
  // return pointer to the begining of usable memory block
  inline Term* get_first() const;
  //
  // return pointer beyond the end of usable memory block
  inline Term* get_last() const;
  //
  // get reference counter
  inline uintptr_t get_ref_count() const;
  //
  // increment reference counter
  inline void inc_ref_count();
  //
  // decrement reference counter,
  // return false if it becomes zero
  inline bool dec_ref_count();
  //
  // return true if there is no references to the chunk from outside ???
  inline bool no_extern_refs() const;
};

inline uintptr_t MemoryChunk::get_size() const
{
  return allocator->get_size(this);
}

inline Term* MemoryChunk::get_first() const
{
  return (Term*)(ptr_add_offset(this, sizeof(MemoryChunk)));
}

inline Term* MemoryChunk::get_last() const
{
  return (Term*)(ptr_add_offset(this, get_size()));
}

inline uintptr_t MemoryChunk::get_ref_count() const
{
  return ref_count;
}

inline void MemoryChunk::inc_ref_count()
{
  //
  // check for reference counter owerflow
  if( !(++ref_count) )
    FATAL( "Reference counter overflow" );
}

inline bool MemoryChunk::dec_ref_count()
{
  return --ref_count;
}

inline bool MemoryChunk::no_extern_refs() const
{
  return ref_count == 1;
}

}

#endif // __rf_memory_chunk_hh__
