//
// Copyright (C) 2000 Refal+ Development Group
//
// Refal+ is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Refal+ is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Refal+; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// $Source$
// $Revision: 2187 $
// $Date: 2006-12-15 03:17:56 +0300 (Fri, 15 Dec 2006) $
// Author: Andrey Slepuhin <pooh@msu.ru>

#ifndef __rf_result_hh__
#define __rf_result_hh__

#include "rf_stack.hh"
#include "rf_expr.ih"

namespace rfrt
{

class Result
{

  friend class Stack ;

private:

  Expr* expr_ptr ;

  Result (Result const&)
  {
    FATAL("Private constructor Result(Result const&) should never be called");
  }

  Result& operator = (Result const&)
  {
    FATAL("Private operator = Result(Result const&) should never be called");
    return *this;
  }

public:

  Result ()
  {
    if (stack->top > stack->last) FATAL("Stack exhausted");
    expr_ptr = stack->top++;
    Expr::clear(expr_ptr);
  }

  ~Result ()
  {}

  Result& operator = (Expr const& _expr)
  {
    new(expr_ptr) Expr(_expr);
    return *this;
  }

  Result& operator , (Result& _res)
  {
    stack->pop(*this);
    return _res;
  }

  Expr& operator , (Expr& _expr)
  {
    stack->pop(*this);
    return _expr;
  }

  Expr const& to_Expr () const
  {
    return *expr_ptr;
  }

};

inline void Stack::pop (Result& _res)
{
  if (_res.expr_ptr != top) {
    new(_res.expr_ptr) Expr(top);
  }
  top++;
}

}

#endif // __rf_result_hh__
